# -*- python -*-
# $RCSfile: chooser.py,v $
# $Revision: 1.52 $
# $Author: langer $
# $Date: 2004/09/08 17:25:41 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from gtk import *
from common import guitop
from common.IO.GUI import gui_utils
from common.IO.GUI import oof_mainiteration
from common import debug
import types

# Calls the callback, if specified, with the gtk-selecting object
# and the name.  Backward compatible with other callbacks that way.
# Also has an "update_callback" which gets called, with the new
# current selection, when the list of things is updated, with one
# exception -- it is *not* called at __init__-time.
class ChooserWidget:
    def __init__(self, namelist, callback=None, callbackargs=(),
                 update_callback=None, update_callback_args=(), helpdict={}):
        debug.mainthreadTest()

        # If this is used as a base class for another widget, self.gtk
        # will be redefined.  So if a ChooserWidget function needs to
        # refer to the OptionMenu gtk widget, it must use
        # self.optionmenu instead of self.gtk.
        self.optionmenu = gui_utils.SizedOptionMenu()
        self.gtk = self.optionmenu
        self.optionmenu.connect('destroy', self.destroyCB)
        self.setsize(namelist)
        self.current_string = None
        self.menu = None
        self.callback = callback
        self.callbackargs = callbackargs
        self.helpdict = {}              # set by self.update()
        
        self.update_callback = None
        self.update_callback_args = ()
        self.update(namelist, helpdict)
        self.update_callback = update_callback
        self.update_callback_args = update_callback_args

        self.signals = {}

    def show(self):
        debug.mainthreadTest()
        self.gtk.show_all()

    def destroyCB(self, *args):
        self.menu.destroy()             # is this necessary?

    def destroy(self):
        debug.mainthreadTest()
        self.gtk.destroy()

    def setsize(self, namelist):
        debug.mainthreadTest()
        # Character length is not quite the right thing, you really
        # want the font length, but we don't know the font.
        charlen = max(map(len, namelist+["dummy"]))
        gui_utils.SizedOptionMenu(gtkoptmenu=self.optionmenu,
                                  charwidth=charlen)

    def update(self, namelist, helpdict={}):
        debug.mainthreadTest()
        # Replace the widget's list of names with the given list.  The
        # original value of self.current_string will be restored if it
        # still exists in the updated namelist.
        self.helpdict = helpdict
        if not self.current_string in namelist: # pick a reasonable new setting
            if len(namelist) > 0:
                self.current_string = namelist[0]
            else:
                self.current_string = None
        self.namelist = namelist
        if self.menu:
            self.optionmenu.remove_menu()
        self.menu = GtkMenu()
        self.optionmenu.set_menu(self.menu)
        self.signals = {}
        for name,i in zip(namelist, range(len(namelist))):
            menuitem = GtkMenuItem(name)
            try:
                tip = helpdict[name]
                guitop.top().tooltips.set_tip(menuitem, tip)
            except KeyError:
                pass
            self.menu.add(menuitem)
            if name == self.current_string:
                self.optionmenu.set_history(i) 
            self.signals[menuitem] = menuitem.connect('activate',
                                                      self.activate, name)
        self.setsize(namelist)
        self.set_tooltip()
        self.optionmenu.show_all()
        if self.update_callback:
            self.update_callback( *( (self.gtk, self.current_string) +
                                     self.update_callback_args) )
                                  
    def activate(self, gtkobj, name):   # gtk callback
        debug.mainthreadTest()
        if name != self.current_string:
            self.current_string = name
            self.set_tooltip()
            if self.callback:
                self.callback( *( (gtkobj, name) + self.callbackargs) )

    def set_tooltip(self):
        if self.current_string is not None:
            try:
                tip = self.helpdict[self.current_string]
                guitop.top().tooltips.set_tip(self.optionmenu, tip)
            except KeyError:
                pass

    def blockSignals(self):
        debug.mainthreadTest()
        for gtkobj, signal in self.signals.items():
            gtkobj.signal_handler_block(signal)
    def unblockSignals(self):
        debug.mainthreadTest()
        for gtkobj, signal in self.signals.items():
            gtkobj.signal_handler_unblock(signal)
    def set_state(self, arg):
        debug.mainthreadTest()
        self.blockSignals()
        if type(arg)==types.IntType:
            self.optionmenu.set_history(arg)
            self.current_string = self.namelist[arg]
        elif type(arg)==types.StringType:
            try:
                i = self.namelist.index(arg)
            except ValueError:
                self.optionmenu.set_history(0)
                self.current_string = self.namelist[0]
            else:
                self.optionmenu.set_history(i)
                self.current_string = arg
        self.set_tooltip()
        self.unblockSignals()
        if self.update_callback:
            self.update_callback( *( (self.gtk, self.current_string) +
                                     self.update_callback_args) )

    def get_value(self):
        return self.current_string

    def nChoices(self):
        return len(self.namelist)

class FakeChooserWidget:
    # For debugging only.
    def __init__(self, namelist, callback=None, callbackargs=(),
                 update_callback=None, update_callback_args=(), helpdict={}):
        self.gtk = GtkLabel("Gen-u-wine Fake Widget")
        debug.fmsg(namelist)
    def show(self):
        pass
    def destroy(self):
        pass
    def setsize(self, namelist):
        pass
    def update(self, namelist, helpdict={}):
        debug.fmsg(namelist)
    def set_state(self, arg):
        pass
    def get_value(self):
        pass
    def nChoices(self):
        pass

## ### #### ##### ###### ####### ######## ####### ###### ##### #### ### ##

# The NewChooserWidget has a pulldown menu and a button.  The default
# text on the button is "New...".  The intent is that the callback for
# the button will create a new object to be listed in the pulldown
# menu. The button callback is responsible for updating the menu (with
# ChooserWidget.update) and selecting the new entry (with
# ChooserWidget.set_state).

class NewChooserWidget(ChooserWidget):
    def __init__(self, namelist, callback=None, callbackargs=(),
                 update_callback=None, update_callback_args=(),
                 button_callback=None, button_callback_args=(),
                 buttontext="New..."):
        ChooserWidget.__init__(self, namelist,
                               callback=callback, callbackargs=callbackargs,
                               update_callback=update_callback,
                               update_callback_args=update_callback_args,
                               helpdict={})

        self.button_callback = button_callback
        self.button_callback_args = button_callback_args
        
        # Wrap the ChooserWidget's gtk in a GtkHBox and make it become self.gtk
        hbox = GtkHBox()
        hbox.add(self.gtk, expand=1, fill=1, padding=2)
        self.gtk = hbox

        self.newbutton = GtkButton(buttontext)
        hbox.add(self.newbutton, expand=0, fill=0, padding=2)
        self.newbutton.connect('clicked', self.newbuttonCB)
    def newbuttonCB(self, button):
        if self.button_callback:
            self.button_callback(*self.button_callback_args)
            

## ### #### ##### ###### ####### ######## ####### ###### ##### #### ### ##    


# Like a ChooserWidget, but makes a GtkList instead of a GtkOptionMenu.
# Locally stateful.  Callback gets called when selection state
# changes, with the newly-selected string or "None".
# Takes both a list of objects, and a list of display strings for
# those objects.  The callback calls back with a selected object.

class ChooserListWidget:
    def __init__(self, objlist=None, displaylist=[], callback=None,
                 dbcallback=None, autoselect=1, helpdict={}):
        debug.mainthreadTest()
        self.gtklist = GtkList()
        # Use self.gtklist locally. A derived class may redefine self.gtk.
        self.gtk = self.gtklist
        self.gtklist.set_selection_mode(SELECTION_SINGLE)
        if callback is None:
            self.callback = lambda x: None
        else:
            self.callback = callback        # single-click callback function
        if dbcallback is None:
            self.dbcallback = lambda x: None
        else:
            self.dbcallback = dbcallback   # double-click callback function
        self.current_obj = None
        self.widgetdict = {}            # Maps widgets to objects.
        self.objdict = {}               # Maps objects to widgets.
        self.displaydict = {}           # Maps objects to displaynames.
        self.autoselect = autoselect    # Automatically select singletons
        self.update(objlist or [], displaylist, helpdict=helpdict)

    # Query function, so you can ask it if has a selection.
    def has_selection(self):
        return self.current_obj is not None
    
    def selectCB(self, gtk):
        self.current_obj = self.widgetdict[gtk]
        if self.callback:
            self.callback(self.current_obj)

    def deselectCB(self, gtk):
        self.current_obj = None
        if self.callback:
            self.callback(None)

    def buttonPressCB(self, listitem, event):
        if event.type == GDK._2BUTTON_PRESS:
            self.current_obj = self.widgetdict[listitem]
            self.dbcallback(self.current_obj)

    def destroy(self):
        debug.mainthreadTest()
        for (w, s, d) in self.objdict.values():
            w.disconnect(s)
            w.disconnect(d)
        self.gtk.destroy()
            
    def get_list(self):
        return self.objlist

    def get_value(self):
        return self.current_obj

    def get_index(self):
        return self.objlist.index(self.current_obj)

    def set_selection(self, obj):
        try:
            w = self.objdict[obj]
        except KeyError:
            pass
        else:
            w[0].select()
    
    # Wholesale replacement of contents.  Preserve selection state, if
    # possible.
    def update(self, objlist, displaylist=[], helpdict={}):
        debug.mainthreadTest()
        # Block the spurious "deselect" signal, if it will be spurious.
        if self.current_obj in objlist:
            (widget, select, deselect) = self.objdict[self.current_obj]
            widget.signal_handler_block(select)
            widget.signal_handler_block(deselect)

        self.gtklist.foreach(GtkObject.destroy)
        self.widgetdict = {}
        self.objdict = {}
        self.displaydict = {}
        for n, dname in map(None, objlist, displaylist):
            if dname is not None:
                self.displaydict[n] = dname
            else:
                self.displaydict[n] = n
            widget = GtkListItem(self.displaydict[n])
            self.gtklist.add(widget)

            try:
                tip = helpdict[dname]
                guitop.top().tooltips.set_tip(widget, tip)
            except KeyError:
                pass
            
            s = widget.connect("select", self.selectCB)
            d = widget.connect("deselect", self.deselectCB)
            widget.connect('button_press_event', self.buttonPressCB)
            self.widgetdict[widget] = n
            self.objdict[n] = (widget, s, d)
        self.objlist = objlist
        # Check it again to do the selection.
        if (self.current_obj in objlist):
            (widget, select, deselect) = self.objdict[self.current_obj]
            widget.select()
        else:
            if len(objlist)==1 and self.autoselect:
                self.current_obj = objlist[0]
                (widget, select, deselect) = self.objdict[self.current_obj]
                widget.select()
            else:
                self.current_obj = None

        self.gtklist.show_all()

    def show(self):
        self.gtk.show_all()
    def hide(self):
        self.gtk.hide()

##################################################

class ChooserComboWidget:
    def __init__(self, namelist, callback=None):
        # If a callback is provided, it's called a *lot* of times.
        # It's called for every keystroke in the entry part of the
        # widget and every time a selection is made or scrolled over
        # in the list part of the widget.
        debug.mainthreadTest()
        self.gtk = GtkCombo()
        self.gtk.set_value_in_list(FALSE, FALSE) # allow new values, no empties

        self.namelist = []
        self.update(namelist)
        self.callback = callback
        if callback:
            self.gtk.list.connect('selection-changed', self.comboCB)
            self.gtk.entry.connect('changed', self.comboCB)

    def show(self):
        debug.mainthreadTest()
        self.gtk.show_all()

    def destroy(self):
        debug.mainthreadTest()
        self.gtk.destroy()

    def comboCB(self, *args):
        self.callback(self)

    def update(self, namelist):
        debug.mainthreadTest()
        current_string = self.get_value()
        if not current_string in namelist:
            if len(namelist) > 0:
                current_string = namelist[0]
            else:
                current_string = ""
        if self.namelist:
            self.gtk.list.clear_items(0, len(self.namelist))
        self.namelist = namelist
        if self.namelist:
            self.gtk.set_popdown_strings(self.namelist)
        self.gtk.entry.set_text(current_string)
        self.gtk.show_all()

    def set_state(self, arg):
        debug.mainthreadTest()
        if type(arg) == types.IntType:
            self.gtk.entry.set_text(self.namelist[arg])
        elif type(arg) == types.StringType:
            self.gtk.entry.set_text(arg)

    def get_value(self):
        val = self.gtk.entry.get_text()
        return val


        
##################################################        

# Used in SkeletonInfo, MeshInfo.
class FramedChooserListWidget(ChooserListWidget):
    def __init__(self, objlist=None, displaylist=[],
                 callback=None, dbcallback=None, autoselect=1):
        ChooserListWidget.__init__(self, objlist=objlist,
                                   displaylist=displaylist,
                                   callback=callback,
                                   dbcallback=dbcallback,
                                   autoselect=autoselect )
        self.gtk = GtkFrame()
        self.gtk.set_shadow_type(SHADOW_IN)
        self.vbox = GtkVBox()
        self.gtk.add(self.vbox)
        self.vbox.add(self.gtklist, expand=1, fill=1)
        self.gtk.show_all()

##################################################

class ScrolledChooserListWidget(ChooserListWidget):
    def __init__(self, objlist=None, displaylist=[], callback=None,
                 dbcallback=None, autoselect=1):
        ChooserListWidget.__init__(
            self, objlist=objlist, displaylist=displaylist, callback=callback,
            dbcallback=dbcallback, autoselect=autoselect)
        self.gtk = GtkScrolledWindow()
        self.gtk.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        self.gtk.add_with_viewport(self.gtklist)
        self.gtk.show_all()

## ### #### ##### ###### ####### ######## ####### ###### ##### #### ### ##



# YAChooser-like-thing.  This one is list-like, but can have
# multiple selections.  Probably ought to be a subclass.

class MultiListWidget:
    def __init__(self, objlist=None, displaylist=[], callback=None):
        debug.mainthreadTest()
        self.list = GtkList()
        self.gtk = self.list
        self.list.set_selection_mode(SELECTION_MULTIPLE)
        self.objlist = objlist or []
        self.callback = callback
        self.widgetdict = {}  # Maps widgets to names.
        self.objdict = {}    # Maps names to widgets.
        self.displaydict = {} # Maps names to displaynames.
        self.selectlist = []

        for name in self.objlist:
            try:
                dname = displaylist[objlist.index(name)]
            except IndexError:
                self.displaydict[name] = name
            else:
                self.displaydict[name] = dname
                
            widget = GtkListItem(self.displaydict[name])
            self.list.add(widget)
            s = widget.connect("select", self.selectCB)
            d = widget.connect("deselect", self.deselectCB)
            self.widgetdict[widget] = name
            self.objdict[name] = (widget, s, d)

    def selectCB(self, gtk):
        self.selectlist.append(self.widgetdict[gtk])
        if self.callback:
            self.callback(self.selectlist)

    def deselectCB(self, gtk):
        self.selectlist.remove(self.widgetdict[gtk])
        if self.callback:
            self.callback(self.selectlist)

    def get_list(self):
        return self.objlist

    # Return a copy of the list, so that subsequent selection-state
    # changes don't mess things up.
    def get_value(self):
        return self.selectlist[:]

    # Set-selection is used to set the selection state in the list.
    # It does not emit signals.  It also does not *deselect* anything.
    def set_selection(self, objlist):
        debug.mainthreadTest()
        for n in objlist:
            try:
                (w, s, d) = self.objdict[n]
            except KeyError:
                pass
            else:
                # Don't re-select already-selected stuff, that
                # messes up the selectlist.
                if self.widgetdict[w] not in self.selectlist:
                    w.signal_handler_block(s)
                    w.select()
                    self.selectlist.append(self.widgetdict[w])
                    w.signal_handler_unblock(s)
            
    def set_value(self, objlist):
        self.set_selection(objlist)

    # Clear all selections, without emitting any signals.
    def clear(self):
        for n in self.selectlist:
            (w,s,d) = self.objdict[n]
            w.signal_handler_block(d)
            w.deselect()
            w.signal_handler_unblock(d)
        self.selectlist = []
            
    # At update-time, names common to the old list and the new list
    # have their selection state preserved. 
    def update(self, objlist, displaylist=[]):
        debug.mainthreadTest()

        old_objlist = self.objlist
        
        # Names common to the old list and the new list have
        # their selection state preserved.  Names in the new
        # list but not in the old list are selected according
        # to the policy.

        # Widget removal emits a "deslect" signal on selected items,
        # which must be blocked for selected widgets present in both
        # lists.  Selected items not in the new list must be recorded,
        # so that the integrity of the selection list is maintained.
        absent_selections = []
        for s in self.selectlist:
            (widget, select, deselect) = self.objdict[s]
            if s in objlist:
                widget.signal_handler_block(select)
                widget.signal_handler_block(deselect)
            else:
                absent_selections.append(s)

        # Remove the selected items which are not in the new
        # list outside the selectlist loop.
        for a in absent_selections:
            (widget, select, deselect) = self.objdict[a]
            widget.deselect()   # Removes it from self.selectlist.
        
        # Remove all the old widgets.
        for w in self.objdict.values():
            name = self.widgetdict[w[0]]
            del self.widgetdict[w[0]]
            del self.objdict[name]
            self.list.remove(w[0])
            w[0].destroy()

        # Add all the new widgets.
        for n in objlist:
            try:
                dname = displaylist[objlist.index(n)]
            except IndexError:
                self.displaydict[n]=n
            else:
                self.displaydict[n]=dname
            widget = GtkListItem(self.displaydict[n])
            self.list.add(widget)
            s = widget.connect("select", self.selectCB)
            d = widget.connect("deselect", self.deselectCB)
            self.widgetdict[widget] = n
            self.objdict[n] = (widget, s, d)
        self.objlist = objlist

        # Reconcile the selection state.  Items in the new list which
        # were selected before should be selected now.
        for o in objlist:
            if o in old_objlist:
                # Selected before, select them now.
                if o in self.selectlist:
                    (widget, select, deselect) = self.objdict[o]
                    widget.signal_handler_block(select)
                    widget.select()
                    widget.signal_handler_unblock(select)

        self.list.show_all()

class ScrolledMultiListWidget(MultiListWidget):
    def __init__(self, objlist=None, displaylist=[], callback=None):
        MultiListWidget.__init__(self, objlist, displaylist, callback)
        mlist = self.gtk
        self.gtk = GtkScrolledWindow()
        self.gtk.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        self.gtk.add_with_viewport(mlist)
        self.gtk.show_all()
        
## ### #### ##### ###### ####### ######## ####### ###### ##### #### ### ##

        
class ChooserDialog:
    def __init__(self, prompt, names, helpdict={}):
        debug.mainthreadTest()
        self.names = names
        self.ok_exit = 0
        self.done = 0

        self.dialog = GtkDialog()
        self.dialog.set_transient_for(guitop.top().gtk)
        self.dialog.connect('destroy', self.cancelCB)
        
        okbutton = GtkButton('OK')
        okbutton.connect('clicked', self.okCB)
        self.dialog.action_area.add(okbutton)
        cancelbutton = GtkButton('Cancel')
        cancelbutton.connect('clicked', self.cancelCB)
        self.dialog.action_area.add(cancelbutton)

        hbox = GtkHBox()
        self.dialog.vbox.add(hbox, padding=5)

        hbox.add(GtkLabel(prompt), expand=0, fill=0, padding=5)
        #
        self.options = ChooserWidget(names, callback=self.optionCB,
                                     helpdict=helpdict)
        hbox.add(self.options.gtk, expand=1, fill=1, padding=5)
        self.options.set_state(0)
        self.current_name = self.options.current_string
        
    def show(self):
        debug.mainthreadTest()
        self.dialog.set_modal(1)
        self.dialog.show_all()
    def close(self):
        debug.mainthreadTest()
        self.dialog.destroy()
    def hide(self):
        debug.mainthreadTest()
        self.dialog.hide()

    def optionCB(self, menuitem, name):
        self.current_name = name
        
    def okCB(self, *args):
        self.ok_exit = 1
        self.done = 1

    def cancelCB(self, *args):
        self.done = 1

def chooser(prompt, names):
    if names:
        dialog = ChooserDialog(prompt, names)
        dialog.show()
        while not dialog.done:
            oof_mainiteration.mainiteration()
        dialog.close()
        if dialog.ok_exit:
            return dialog.current_name
        return None                     # cancelled
    else:
        raise "No names!"
