# -*- python -*-
# $RCSfile: layerwidget.py,v $
# $Revision: 1.21 $
# $Author: reida $
# $Date: 2004/09/01 17:44:06 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

# Widget for a single line in the display listing at the bottom of the
# graphics window.

from SWIG.common import switchboard
from common import debug
from common import guitop
from common.IO import layereditor
from gtk import *
import string

class LayerWidget:
    def __init__(self, gfxwindow, displaymethod):
        debug.mainthreadTest()
        self.gfxwindow = gfxwindow
        self.displaymethod = displaymethod
        gfxwindow.addLayerWidget(displaymethod, self)
        self.gtk = GtkListItem()    # a GtkBin derived class
        self.gtk.connect('destroy', self.destroyCB)
        self.sbcallbacks = []

    ######################

    def destroy(self, *args):
        debug.mainthreadTest()
        self.gtk.signal_handler_block(self.deselecthandlerID)
        self.gtk.destroy()

    def destroyCB(self, *args):         # gtk destroy callback
        map(switchboard.removeCallback, self.sbcallbacks)
        self.gfxwindow.removeLayerWidget(self.displaymethod)
        self.gfxwindow = None
        self.displaymethod = None
        self.gtk = None
        
    def who(self):
        return self.displaymethod.who()



class ListedLayerWidget(LayerWidget):
    def __init__(self, gfxwindow, displaymethod):
        LayerWidget.__init__(self, gfxwindow, displaymethod)
        self.selected = 0
        self.doubleclicked = 0 # suppresses superfluous second selection
        self.lastop = 'deselect'
        self.sbcallbacks.append(
            switchboard.requestCallback('rename who', self.renameWho))
        
        self.selecthandlerID = self.gtk.connect('select', self.selectCB)
        self.deselecthandlerID = self.gtk.connect('deselect', self.deselectCB)
        self.gtk.connect('toggle', self.toggleCB)

        box = GtkHBox()
        self.gtk.add(box)

        self.raiseButton = GtkButton()
        self.raiseButton.add(GtkArrow(at=ARROW_UP))
        box.pack_start(self.raiseButton, expand=0)
        self.raiseButton.connect('clicked', self.raiseButtonCB)
        guitop.top().tooltips.set_tip(self.raiseButton, "Raise this layer.")

        self.lowerButton = GtkButton()
        self.lowerButton.add(GtkArrow(at=ARROW_DOWN))
        box.pack_start(self.lowerButton, expand=0)
        self.lowerButton.connect('clicked', self.lowerButtonCB)
        guitop.top().tooltips.set_tip(self.lowerButton, "Lower this layer.")

        self.showButton = GtkCheckButton("")
        self.showButtonLabel = self.showButton.children()[0]

        # "Show" and "Hide" strings are repeated in the
        # "setShowButton" routine below.
        if displaymethod.hidden:
            self.showButton.set_active(0)
            self.showButtonLabel.set_text("Show")
        else:
            self.showButton.set_active(1)
            self.showButtonLabel.set_text("Hide")

        # print "Layer ", self, "connecting to button ", self.showButton
        self.showhandlerID=self.showButton.connect('clicked',
                                                   self.showButtonCB)
        box.pack_start(self.showButton, expand=0)
        guitop.top().tooltips.set_tip(self.showButton,
                                      'Show or hide this layer.')


        # Contourmap label strings are repeated in the
        # "setContourmapButton" routine below.
        self.contourmapButton = GtkCheckButton("Show contourmap")
        self.contourmapButtonLabel = self.contourmapButton.children()[0]
        self.contourmapButton.set_active(0)
        if displaymethod.contour_capable(gfxwindow) \
               and not self.displaymethod.hidden:
            self.contourmapButton.set_sensitive(1)
            if displaymethod.contourmaphidden==0: # If not hidden...
                self.contourmapButton.set_active(1)
                self.contourmapButtonLabel.set_text("Hide contourmap")
        else:
            self.contourmapButton.set_sensitive(0)
        self.contourmaphandlerID=self.contourmapButton.connect(
            'clicked', self.contourmapButtonCB)
        box.pack_start(self.contourmapButton, expand=0)
        guitop.top().tooltips.set_tip(self.contourmapButton,
                                      'Show the contourmap for this layer.')


        self.label = GtkLabel()
        self.updateText()
        self.label.set_alignment(0.0, 0.5)
        box.pack_start(self.label, expand=1, fill=1, padding=5)

        # For catching doubleclicks anywhere on the line
        self.gtk.connect('button_press_event', self.buttonPressCB)

        self.gtk.show_all()

    def updateText(self):
        debug.mainthreadTest()
        if self.gfxwindow.settings.longlayernames:
            self.label.set_text("%s(%s) %s" %
                                (self.who().classname, self.who().name(),
                                 `self.displaymethod`))
        else:
            self.label.set_text("%s(%s) %s" %
                                (self.who().classname, self.who().name(),
                                 self.displaymethod.short_name() ))

    def renameWho(self, classname, oldpath, newname):
        if self.who() and self.displaymethod.who().classname == classname:
            self.updateText()

    def replaceMethod(self, method):
        self.gfxwindow.removeLayerWidget(self.displaymethod)
        self.displaymethod = method
        self.gfxwindow.addLayerWidget(method, self)
        self.updateText()

    #######################

    def showButtonCB(self, button):
        # print "ShowButtonCB of widget ", self
        if button.get_active():
            self.displaymethod.show()
            self.gfxwindow.layerShownByGUI(self.displaymethod)
        else:
            self.displaymethod.hide()
            self.gfxwindow.layerHiddenByGUI(self.displaymethod)

    # Button callback calls the appropriate menu item directly.
    def contourmapButtonCB(self, button):
        if button.get_active():
            menuitem = self.gfxwindow.menu.Layer.Show_Contour_Map
        else:
            menuitem = self.gfxwindow.menu.Layer.Hide_Contour_Map
        n=self.gfxwindow.layerID(self.displaymethod)
        menuitem(n=n)
            

    # Explicit label strings for both of these routines are repeated
    # in the __init__ where the buttons are defined.
    def setShowButton(self, state):     # called by GfxWindow
        debug.mainthreadTest()
        self.showButton.signal_handler_block(self.showhandlerID)
        self.showButton.set_active(state)
        self.showButton.signal_handler_unblock(self.showhandlerID)
        if state:
            self.showButtonLabel.set_text("Hide")
        else:
            self.showButtonLabel.set_text("Show")

    def setContourmapButton(self, state):
        debug.mainthreadTest()
        self.contourmapButton.signal_handler_block(self.contourmaphandlerID)
        self.contourmapButton.set_active(state)
        self.contourmapButton.signal_handler_unblock(self.contourmaphandlerID)
        if state:
            self.contourmapButtonLabel.set_text("Hide contourmap")
        else:
            self.contourmapButtonLabel.set_text("Show contourmap")


    ##########################

    def selectCB(self, *args):
        if not (self.doubleclicked and self.lastop == 'select'):
            self.gfxwindow.layerSelectedByGUI(self.displaymethod)
            self.doubleclicked = 0
            self.lastop = 'select'
    def deselectCB(self, *args):
        # This can occur after the window is closed?!
        if self.gfxwindow and self.gfxwindow.gtk:
            self.gfxwindow.layerDeselectedByGUI(self.displaymethod)
            self.doubleclicked = 0
            self.lastop = 'deselect'

    def select(self):                   # called by GfxWindow
        debug.mainthreadTest()
        self.gtk.signal_handler_block(self.selecthandlerID)
        self.gtk.select()
        self.selected = 1
        self.gtk.signal_handler_unblock(self.selecthandlerID)
        
    def deselect(self):
        debug.mainthreadTest()
        if self.gtk:                    # can occur after object is destroyed?
            self.gtk.signal_handler_block(self.deselecthandlerID)
            self.gtk.deselect()
            self.selected = 0
            self.gtk.signal_handler_unblock(self.deselecthandlerID)
        
    def toggleCB(self, *args):
        if self.gfxwindow:
            if self.selected:
                self.deselectCB()
            else:
                self.selectCB()

    def buttonPressCB(self, listitem, event):
        if event.type == GDK._2BUTTON_PRESS:
            self.doubleclicked = 1
            listitem.select()
            layereditor.openLayerEditor()
            layereditor.layerEditor.menu.LayerSet.Edit(
                window=self.gfxwindow.name,
                layerNumber=self.gfxwindow.layerID(self.displaymethod))
        
    ############################

    def lowerButtonCB(self, *args):
        if self.gfxwindow:
            self.gfxwindow.lowerLayerByGUI(self.displaymethod)
    def raiseButtonCB(self, *args):
        if self.gfxwindow:
            self.gfxwindow.raiseLayerByGUI(self.displaymethod)
    def disableLowerButton(self):
        self.lowerButton.set_sensitive(0)
    def disableRaiseButton(self):
        self.raiseButton.set_sensitive(0)
    def enableRaiseButton(self):
        self.raiseButton.set_sensitive(1)
    def enableLowerButton(self):
        self.lowerButton.set_sensitive(1)

#################

# Layers that aren't listed in the gfxwindow's layer list still have a
# widget which marks their place in the list, so that the other
# widgets' raise and lower buttons make some sense.  The
# UnListedLayerWidget just puts a horizontal line in the list.

class UnListedLayerWidget(LayerWidget):
    def __init__(self, gfxwindow, displaymethod):
        LayerWidget.__init__(self, gfxwindow, displaymethod)
        self.gtk.add(GtkHSeparator())
        self.gtk.set_sensitive(0)
        self.gtk.show_all()
        
    def renameWho(self, *args): pass

    def setShowButton(self, state): pass
    def setContourmapButton(self, state): pass

    def select(self): pass
    def deselect(self): pass

    def disableRaiseButton(self): pass
    def disableLowerButton(self): pass
    def enableRaiseButton(self): pass
    def enableLowerButton(self): pass

    def updateText(self): pass
