# -*- python -*-
# $RCSfile: menudialog.py,v $
# $Revision: 1.11 $
# $Author: langer $
# $Date: 2004/09/02 18:52:41 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

# Create a dialog box for an OOF menu containing commands but no
# submenus or variables (ie, no CheckOOFMenuItems or
# RadioOOFMenuItems).  Clicking on the 'OK' button executes the
# selected command.

from SWIG.common import switchboard
from common import debug
from common import guitop
from common import utils
from common.IO.GUI import gfxmenu
from common.IO.GUI import gui_utils
from common.IO.GUI import parameterwidgets
from common.IO.GUI import widgetscope
from common.IO.GUI import oof_mainiteration
from gtk import *

class MenuDialog(widgetscope.WidgetScope):
    def __init__(self, menu, scope=None, title=None, presets=[]):
        # presets are Parameters that are already set and should not
        # appear in the widget.
        debug.mainthreadTest()
        widgetscope.WidgetScope.__init__(self, scope)
        self.done = 0
        self.presets = presets

        self.dialog = GtkDialog()
        self.dialog.connect('destroy', self.cancelCB)

        if len(menu.items) == 0:
            self.done = 1
            return
        
        self.okbutton = GtkButton('OK')
        self.okbutton.connect('clicked', self.okCB)
        self.dialog.action_area.add(self.okbutton)
        cancelbutton = GtkButton('Cancel')
        cancelbutton.connect('clicked', self.cancelCB)
        self.dialog.action_area.add(cancelbutton)

        if title:
            self.dialog.set_title(title)
            hbox = GtkHBox()
            self.dialog.vbox.add(hbox, expand=0, fill=0, padding=5)
            hbox.add(GtkLabel(title), expand=1, fill=1, padding=10)

        self.options = gui_utils.SizedOptionMenu()
        gtkmenu = GtkMenu()
        self.options.set_menu(gtkmenu)
        self.menubox = GtkHBox()
        self.dialog.vbox.add(self.menubox, expand=0, fill=0, padding=5)
        self.menubox.add(self.options, expand=1, fill=1, padding=5)

        for item in menu.items:
            name = utils.underscore2space(item.name)
            menuitem = GtkMenuItem(name)
            if item.helpstr:
                guitop.top().tooltips.set_tip(menuitem, item.helpstr)
            gtkmenu.append(menuitem)
            menuitem.connect('activate', self.optionCB, item)

        accelgroup = GtkAccelGroup()
        self.dialog.add_accel_group(accelgroup)
        self.okbutton.add_accelerator('clicked', accelgroup, GDK.Return,
                                      0, ACCEL_VISIBLE)

        self.currentItem = None
        self.options.set_history(0)
        self.optionCB(None, menu.items[0])

    def optionCB(self, gtkitem, menuitem):
        # menuitem is the OOFMenuItem
        if self.currentItem is not None and self.paramWidget is not None:
            switchboard.removeCallback(self.sbcallback)
            self.paramWidget.get_values()
            self.paramWidget.destroy()
        self.currentItem = menuitem
        params = [p for p in menuitem.params if p not in self.presets]
        self.paramWidget = parameterwidgets.ParameterTable(params,
                                                           scope=self)
        self.dialog.vbox.pack_start(self.paramWidget.gtk)
        self.show_pieces()
        self.sbcallback = switchboard.requestCallback(('validity',
                                                       self.paramWidget),
                                                      self.validityCB)
        if menuitem.helpstr:
            guitop.top().tooltips.set_tip(self.options, menuitem.helpstr)
        else:
            guitop.top().tooltips.set_tip(self.options, "")
        self.sensitize()

    def sensitize(self):
        if self.paramWidget and self.paramWidget.isValid():
            self.okbutton.set_sensitive(1)
        else:
            self.okbutton.set_sensitive(0)

    def validityCB(self, validity):
        self.sensitize()

    def show_pieces(self):
        # Don't just use self.dialog.show_all(), since some of the
        # widgets in the ParameterTable might not want to be shown. 
        debug.mainthreadTest()
        self.dialog.action_area.show_all()
        self.dialog.vbox.show()
        self.menubox.show_all()
        if self.paramWidget is not None:
            self.paramWidget.show()

    def show(self):
        debug.mainthreadTest()
        self.done = 0
        self.dialog.set_modal(1)
        self.show_pieces()
        self.dialog.show()
    def close(self):
        debug.mainthreadTest()
        switchboard.removeCallback(self.sbcallback)
        self.dialog.destroy()
        self.destroyScope()
    def hide(self):
        debug.mainthreadTest()
        self.dialog.hide()
    def okCB(self, *args):
        self.done = 1
        if self.currentItem is not None:
            self.paramWidget.get_values()
            self.currentItem.callWithDefaults()
    def cancelCB(self, *args):
        self.done = 1

_dialogs = {}

def invokeMenuDialog(menu, *args, **kwargs): # menu must be an OOFMenuItem
    dialog = MenuDialog(menu, *args, **kwargs)
    dialog.show()
    while not dialog.done:
        oof_mainiteration.mainiteration()
    dialog.close()
