# -*- python -*-
# $RCSfile: microstructureIPC.py,v $
# $Revision: 1.12 $
# $Author: redwing $
# $Date: 2004/07/21 21:01:18 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 


from common import debug
from common import primitives
from common.IO import oofmenu
from common.IO import parallelmainmenu
from common.IO import parameter
from common.IO import socket2me
from SWIG.common import mpitools


_rank = mpitools.rank()
_size = mpitools.size()
## OOF.LoadData.IPC.Microstructure
ipcmicrostructuremenu = parallelmainmenu.ipcmenu.addItem(oofmenu.OOFMenuItem('Microstructure', secret=1, no_log=1))


## OOF.LoadData.IPC.Microstructure.Initialize
## Transfers the microstructure to the back-end

def _parallel_init_mic(menuitem, microstructure, pixelsize, physicalsize):
    debug.fmsg()
    ## microstructure is the name of the microstructure
    global _rank
    import common.microstructure
    ## front-end work
    if _rank == 0:
        pass            
    ## back-end part of the work
    else:
        common.microstructure.Microstructure(microstructure, pixelsize, physicalsize)

        



ipcmicrostructuremenu.addItem(oofmenu.OOFMenuItem('Initialize', callback = _parallel_init_mic, secret=1, no_log=1, threadable = oofmenu.PARALLEL_THREADABLE, params=[parameter.StringParameter('microstructure'), primitives.iPointParameter('pixelsize'), primitives.PointParameter('physicalsize')] ))


def _add_img_context(menuitem, microstructure, imagecontext):
## arguments are microstructure name and image context name
    global _rank
    if _rank == 0:
        ## front-end sends imagecontext name and microstructure name
        pass
    else:
        debug.fmsg()
        ## back-end resolves the ImageContext, and adds it to the right
        ## microstructure
        ## assume that img_context already exists
        from SWIG.image import oofimage ## avoid import loop
        the_name = microstructure + ":" + imagecontext
        img_context = oofimage.imageContexts[the_name]
        import common.microstructure
        ms = common.microstructure.microStructures[microstructure].getMicrostructure() ## actual microstructure
        ms.addImage(img_context) ## adds image context to microstructure on the other side
    ## image.addMicrostructure will be set automatically once the name is resolved


## OOF.LoadData.Microstructure.AddImageContext
ipcmicrostructuremenu.addItem(oofmenu.OOFMenuItem('AddImageContext', callback = _add_img_context, secret=1, no_log=1, threadable = oofmenu.PARALLEL_THREADABLE, params=[parameter.StringParameter('microstructure'), parameter.StringParameter('imagecontext')] ))
