# -*- python -*-
# $RCSfile: microstructuremenu.py,v $
# $Revision: 1.40 $
# $Author: langer $
# $Date: 2004/08/08 02:27:38 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 


from SWIG.image import oofimage
from common import enum
from common import debug
from common import primitives
from common import quit
from common.IO import automatic
from common.IO import datafile
from common.IO import mainmenu
from common.IO import microstructureIO
from common.IO import oofmenu
from common.IO import parameter
from common.IO import whoville
import common.microstructure
from types import *

micromenu = mainmenu.OOF.addItem(oofmenu.OOFMenuItem('Microstructure',
                                                     cli_only=1))
#######################

def newMicrostructure(menuitem,
                      name, width, height, widthInPixels, heightInPixels):
    ms = common.microstructure.Microstructure(name,
                                              primitives.iPoint(widthInPixels,
                                                                heightInPixels),
                                              primitives.Point(width, height))

def msNameResolver(param, name):
    if param.automatic():
        nm = 'microstructure'
    else:
        nm = name
    return common.microstructure.microStructures.uniqueName(nm)
    
micromenu.addItem(
    oofmenu.OOFMenuItem('New',
                        callback=newMicrostructure,
                        params=[
    parameter.AutomaticNameParameter('name', 
                                     resolver=msNameResolver,
                                     value=automatic.automatic,
                                     tip="Name of the new Microstructure."),
    parameter.FloatParameter('width', 1., tip='Width in physical units'),
    parameter.FloatParameter('height', 1.,tip='Height in physical units.'),
    parameter.IntParameter('widthInPixels', 10, tip='Width in pixels.'),
    parameter.IntParameter('heightInPixels', 10, tip='Height in pixels.')
    ],
                        help="Create a new Microstructure."
                        )
    )

############################

def renameMicrostructure(menuitem, microstructure, name):
    ms = common.microstructure.microStructures[microstructure]
    ms.begin_writing()
    ms.rename(name, exclude=ms.getObject().name())
    ms.end_writing()

micromenu.addItem(
    oofmenu.OOFMenuItem('Rename',
                        callback=renameMicrostructure,
                        params=[
    parameter.StringParameter('microstructure', '',
                              tip='Old name for the microstructure'),
    parameter.StringParameter('name', '',
                              tip='New name for the microstructure')],
                        help="Rename a Microstructure."))

############################

def copyMicrostructure(menuitem, microstructure, name):
    ms = common.microstructure.microStructures[microstructure]
    ms.begin_writing()
    sourceMS = ms.getObject()
    newMS = sourceMS.nominalCopy(name)
    # First, copy images and load them to copied MS.
    for image in sourceMS.images:
        sourceimage = image.getObject()
        immidge = sourceimage.clone(sourceimage.name())
        imagecontext = oofimage.imageContexts.add(
            [newMS.name(),immidge.name()], immidge)
        newMS.addImage(imagecontext)
    # Copy pixel groups
    for grpname in sourceMS.groupNames():
        sourcegrp = sourceMS.findGroup(grpname)
        newgrp = newMS.getGroup(grpname)
        newgrp.add(sourcegrp.members())
        newgrp.set_meshable(sourcegrp.is_meshable())
    ms.end_writing()

micromenu.addItem(
    oofmenu.OOFMenuItem('Copy',
                        callback=copyMicrostructure,
                        params=[
    parameter.StringParameter('microstructure', '',
                              tip='The source microstructure'),    
    parameter.AutomaticNameParameter('name', 
                                     resolver=msNameResolver,
                                     value=automatic.automatic,
                                     tip="Name of the new Microstructure.")],
                        help="Copy a Microstructure."
                        )
    )

#############################

def deleteMicrostructure(menuitem, microstructure):
    mscontext = common.microstructure.microStructures[microstructure] # who obj
    mscontext.begin_writing()
    try:
        ms = mscontext.getObject()      # ms obj
        common.microstructure.microStructures.remove(microstructure)
        ms.destroy()
    finally:
        mscontext.end_writing()

micromenu.addItem(
    oofmenu.OOFMenuItem('Delete',
                        callback=deleteMicrostructure,
                        params=[
    parameter.StringParameter('microstructure', '',
                              tip='name of the microstructure to be deleted')],
                        help="Destroy a Microstructure."))


#########################

def saveMicrostructure(menuitem, filename, mode, format, microstructure):
    quit.disallow()
    try:
        ms = common.microstructure.microStructures[microstructure]
        dfile = datafile.writeDataFile(filename, mode, format)
        microstructureIO.writeMicrostructure(dfile, ms)
        dfile.close()
    finally:
        quit.allow()

mainmenu.OOF.File.Save.addItem(oofmenu.OOFMenuItem(
    'Microstructure',
    callback=saveMicrostructure,
    params=[
    parameter.StringParameter('filename'),
    parameter.StringParameter('mode'),
    enum.EnumParameter('format', datafile.DataFileFormat, datafile.ASCII),
    whoville.WhoParameter('microstructure',
                          common.microstructure.microStructures)],
    help="Save a Microstructure in a file."
   ))


