# -*- python -*-
# $RCSfile: boundaryAnalysisPage.py,v $
# $Revision: 1.4 $
# $Author: haan $
# $Date: 2004/09/15 20:31:09 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from SWIG.common import switchboard
from common import guitop
from common.IO.GUI import oofGUI
from common.IO.GUI import whowidget
from engine.IO import meshbdymenu
from engine.IO.GUI import skeletongroupwidgets # For boundary widget.
import engine.mesh
import gtk

# Page for computing the flux through a boundary.  Not obvious that
# this needs its own page, but also not obvious where else to put it.


class BoundaryData(oofGUI.MainPage):
    def __init__(self):
        oofGUI.MainPage.__init__(self, name="Boundary Analysis",
                                 ordering=271,
                                 tip="Examine the boundaries of the system.")
        
        mainbox = gtk.GtkVBox()
        self.gtk.add(mainbox)

        hbox = gtk.GtkHBox()
        mainbox.add(hbox, expand=0, fill=0)
        centerbox = gtk.GtkHBox(spacing=3)
        hbox.add(centerbox, expand=1, fill=0)
        self.meshwidget = whowidget.WhoWidget(engine.mesh.meshes,
                                              callback=self.meshCB,
                                              scope=self)
        label = gtk.GtkLabel("Microstructure=")
        label.set_alignment(1.0, 0.5)
        centerbox.add(label, expand=0, fill=0)
        centerbox.add(self.meshwidget.gtk[0], expand=0, fill=0)
        label = gtk.GtkLabel("Skeleton=")
        label.set_alignment(1.0, 0.5)
        centerbox.add(label, expand=0, fill=0)
        centerbox.add(self.meshwidget.gtk[1], expand=0, fill=0)
        label = gtk.GtkLabel("Mesh=")
        label.set_alignment(1.0, 0.5)
        centerbox.add(label, expand=0, fill=0)
        centerbox.add(self.meshwidget.gtk[2], expand=0, fill=0)

        # Make a frame for the two obvious params.
        dataframe = gtk.GtkFrame("Parameters")
        mainbox.add(dataframe, expand=1, fill=1)

        databox = gtk.GtkVBox()
        dataframe.add(databox)

        self.menuitem = meshbdymenu.meshbdymenu.Integrate_Flux

        self.meshparam = self.menuitem.get_arg('mesh')
        self.boundaryparam = self.menuitem.get_arg('boundary')
        self.fluxparam = self.menuitem.get_arg('flux')
        self.destparam = self.menuitem.get_arg('destination')

        self.table = gtk.GtkTable()

        self.table.attach(gtk.GtkLabel("boundary = "),
                          0, 1, 0, 1, xoptions=0)
        self.table.attach(gtk.GtkLabel("flux = "),
                          0, 1, 1, 2, xoptions=0)
        
        self.boundarywidget = self.boundaryparam.makeWidget(scope=self)
        self.fluxwidget = self.fluxparam.makeWidget(scope=self)


        self.table.attach(self.boundarywidget.gtk,
                          1, 2, 0, 1)
        self.table.attach(self.fluxwidget.gtk,
                          1, 2, 1, 2)
        databox.add(self.table, expand=0, fill=0)
        
        goframe = gtk.GtkFrame("Destination")
        gobox = gtk.GtkHBox()
        goframe.add(gobox)
        
        self.destwidget = self.destparam.makeWidget()
        gobox.pack_start(self.destwidget.gtk, expand=1, fill=1, padding=2)

        self.gobutton = gtk.GtkButton("Go!")
        self.gobutton.connect("clicked", self.goCB)
        guitop.top().tooltip(self.gobutton, "Send the output to the destination.")
        gobox.pack_end(self.gobutton,expand=1,fill=1, padding=2)

        mainbox.add(goframe,expand=0,fill=0)

        switchboard.requestCallback("equation activated",
                                    self.equationCB)
        switchboard.requestCallback( ("validity", self.destwidget),
                                     self.sensitize)

        self.sensitize()
        
    def meshCB(self, *args):
        self.fluxwidget.update()
        self.sensitize()

    def equationCB(self, meshname, eqname, activity):
        if meshname==self.meshwidget.get_value():
            self.fluxwidget.update()
            self.sensitize()

    def sensitize(self, *args):
        go_sense = 0
        if self.fluxwidget.isValid() and self.boundarywidget.isValid() \
           and self.destwidget.isValid() and self.meshwidget.get_value():
            go_sense=1
        self.gobutton.set_sensitive(go_sense)
        
    def goCB(self, gtkobj):
        self.meshparam.value = self.meshwidget.get_value()
        self.boundaryparam.value = self.boundarywidget.get_value()
        self.fluxparam.value = self.fluxwidget.get_value()
        self.destparam.value = self.destwidget.get_value()
        self.menuitem.callWithDefaults()
    
boundaryAnalysisPage = BoundaryData()


