# -*- python -*-
# $RCSfile: equationPage.py,v $
# $Revision: 1.15 $
# $Author: langer $
# $Date: 2004/07/09 21:09:12 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from SWIG.common import switchboard
from SWIG.engine import equation
from common import debug
from common import guitop
from common import labeltree
from common import microstructure
from common.IO import mainmenu
from common.IO.GUI import oofGUI
from common.IO.GUI import whowidget
from common.IO.GUI import parameterwidgets
from engine import skeletoncontext
from gtk import *
import engine.mesh

meshmenu = mainmenu.OOF.Mesh

class ButtonSignal:
    def __init__(self, button, signal):
        self.button = button
        self.signal = signal
    def set(self, active):
        debug.mainthreadTest()
        self.button.signal_handler_block(self.signal)
        self.button.set_active(active)
        self.button.signal_handler_unblock(self.signal)
    def set_sensitive(self, sensitivity):
        debug.mainthreadTest()
        self.button.set_sensitive(sensitivity)


class EquationPage(oofGUI.MainPage):
    def __init__(self):
        oofGUI.MainPage.__init__(self, name="Equations", ordering=220,
                                 tip="Define equations for the mesh to solve.")
        mainbox = GtkVBox()
        self.gtk.add(mainbox)

        hbox = GtkHBox()
        mainbox.add(hbox, expand=0, fill=0)
        centerbox = GtkHBox(spacing=3)
        hbox.add(centerbox, expand=1, fill=0)
        self.meshwidget = whowidget.WhoWidget(engine.mesh.meshes,
                                              callback=self.meshCB, scope=self)
        label = GtkLabel("Microstructure=")
        label.set_alignment(1.0, 0.5)
        centerbox.add(label, expand=0, fill=0)
        centerbox.add(self.meshwidget.gtk[0], expand=0, fill=0)
        label = GtkLabel("Skeleton=")
        label.set_alignment(1.0, 0.5)
        centerbox.add(label, expand=0, fill=0)
        centerbox.add(self.meshwidget.gtk[1], expand=0, fill=0)
        label = GtkLabel("Mesh=")
        label.set_alignment(1.0, 0.5)
        centerbox.add(label, expand=0, fill=0)
        centerbox.add(self.meshwidget.gtk[2], expand=0, fill=0)
        
        eqnframe = GtkFrame("Equations")
        scroll = GtkScrolledWindow()
        scroll.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        eqnframe.add(scroll)
        bbox = GtkVBox() # extra layer keeps table from expanding inside scroll
        scroll.add_with_viewport(bbox)
        self.eqntable = GtkTable()
        bbox.add(self.eqntable, expand=0, fill=0)
        self.eqnbuttons = {}
        self.build_eqnTable()
        switchboard.requestCallback("new equation", self.build_eqnTable)
        switchboard.requestCallback("equation activated", self.activateEqnCB)
        mainbox.add(eqnframe)
        
        copybox = GtkHBox(spacing=3)
        self.copybutton = GtkButton("Copy Equation State")
        self.copybutton.connect("clicked", self.copyeqstateCB)
        copybox.add(self.copybutton, expand=1,fill=0)
        mainbox.add(copybox, expand=0, fill=0)
        guitop.top().tooltips.set_tip(self.copybutton,
                                      "Copy the status of all equations from the current mesh to another mesh.")

        self.sensitize()

        switchboard.requestCallback(("new who", "Skeleton"), self.newSkeleton)
        switchboard.requestCallback(("new who", "Mesh"), self.newMesh)
        switchboard.requestCallback(("remove who", "Mesh"), self.removeMesh)
        switchboard.requestCallback("made reservation", self.reservationChanged)
        switchboard.requestCallback("cancelled reservation",
                                    self.reservationChanged)

    def show(self):
        debug.mainthreadTest()
        self.gtk.show_all()
        self.update()

    #######################

    def build_eqnTable(self):
        debug.mainthreadTest()
        self.eqntable.foreach(GtkObject.destroy) # clear the table
        self.eqnbuttons = {}
        eqlist = equation.allEquations
        self.eqntable.resize(len(eqlist), 3)
        row=0
        for eqn in eqlist:
            label = GtkLabel(eqn.name())
            label.set_alignment(1.0, 0.5)
            self.eqntable.attach(label, 0,1, row, row+1,
                                 xoptions=FILL, yoptions=0)
            button = GtkCheckButton('active')
            signal = button.connect('clicked', self.eqnButtonCB, eqn)
            self.eqnbuttons[eqn.name()] = ButtonSignal(button, signal)
            guitop.top().tooltips.set_tip(button,
                                          'Active equations will be solved.')
            self.eqntable.attach(button, 2,3, row,row+1, xoptions=0,
                                 yoptions=0)
            row += 1
        self.eqntable.attach(GtkVSeparator(), 1,2, 0,len(eqlist),
                             xoptions=0, yoptions=EXPAND|FILL)
        self.eqntable.set_col_spacing(0, 3)

    def eqnButtonCB(self, button, eqn): # gtk callback
        if button.get_active():
            meshmenu.Equation.Activate(
                mesh=self.currentFullMeshName(), equation=eqn)
        else:
            meshmenu.Equation.Deactivate(
                mesh=self.currentFullMeshName(), equation=eqn)

    def activateEqnCB(self, mesh, eqn, active): # switchboard "equation activated"
        if mesh == self.currentFullMeshName():
            self.eqnbuttons[eqn].set(active)

    ########################

    def currentMSName(self):
        path = labeltree.makePath(self.currentFullMeshName())
        return path[0]
    def currentMS(self):
        msname = self.currentMSName()
        if msname:
            return microstructure.microStructures[msname].getObject()

    def currentSkeletonName(self):
        path = labeltree.makePath(self.currentFullMeshName())
        return path[1]
    def currentSkeleton(self):
        ms = self.currentMS()
        meshname = self.currentSkeletonName()
        if meshname:
            return skeletoncontext.getSkeleton(ms, meshname)

    def currentFullMeshName(self):
        return self.meshwidget.get_value()
    def currentMeshName(self):
        path = labeltree.makePath(self.currentFullMeshName())
        return path[2]
    def currentMesh(self):
        try:
            return engine.mesh.meshes[self.currentFullMeshName()].getObject()
        except KeyError:
            return None
    def currentMeshContext(self):
        try:
            return engine.mesh.meshes[self.currentFullMeshName()]
        except KeyError:
            return None
            
    def sensitize(self):
        debug.mainthreadTest()
        msok = self.currentMS() is not None
        skelok = msok and self.currentSkeleton() is not None
        meshok = skelok and self.currentMesh() is not None and \
                 not self.currentMeshContext().query_reservation()
        self.eqntable.set_sensitive(meshok)
        self.copybutton.set_sensitive(meshok)
        
    def update(self):
        self.set_state(self.currentFullMeshName())
        
    def set_state(self, meshpath):
        path = labeltree.makePath(meshpath)
        self.meshwidget.set_value(path)
        if path[-1]:                    # ie, mesh name is not empty
            themesh = engine.mesh.meshes[path]
            skel = themesh.skeleton

            for eqn in equation.allEquations:
               active = themesh.getObject().is_active_equation(eqn)
               self.eqnbuttons[eqn.name()].set(active)
            
        else:                           # no current mesh
            for button in self.eqnbuttons.values():
                button.set(0)

    def newSkeleton(self, skeletonname): # switchboard ("new who", "Skeleton")
        self.sensitize()

    def copyeqstateCB(self, gtkobj):
        menuitem = meshmenu.Copy_Equation_State
        targetparam = menuitem.get_arg("target")
        targetparam.set(self.currentFullMeshName())
        if parameterwidgets.getParameters(targetparam,
                                          title="Select a target mesh"):
            menuitem.callWithDefaults(source=self.currentFullMeshName())
    
    def newMesh(self, meshname):        # switchboard ("new who", "Mesh")
        self.set_state(meshname)
        self.sensitize()
        
    def meshCB(self, meshpath):         # WhoWidget callback
        self.set_state(meshpath)
        self.sensitize()

    def removeMesh(self, path):    # switchboard ("remove who", "Mesh")
        self.update()
        self.sensitize()

    def reservationChanged(self, whoobj):
        # switchboard "made reservation", "cancelled reservation"
        if  self.currentFullMeshName() == whoobj.path():
            self.sensitize()


#############
        
ep = EquationPage()
