
# -*- python -*-
# $RCSfile: materialsPage.py,v $
# $Revision: 1.72 $
# $Author: reida $
# $Date: 2004/09/21 14:31:23 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from SWIG.common import ooferror
from SWIG.common import switchboard
from common import debug
from common import guitop
from common import microstructure
from common.IO import mainmenu
from common.IO import mainmenu
from common.IO import reporter
from common.IO.GUI import chooser
from common.IO.GUI import gfxLabelTree
from common.IO.GUI import gui_utils
from common.IO.GUI import oofGUI
from common.IO.GUI import parameterwidgets
from common.IO.GUI import whowidget
from engine import propertyregistration
from engine.IO.GUI import materialListGUI

AllProperties = propertyregistration.AllProperties


from gtk import *

import types, string

# Define some convenience variables.
OOF = mainmenu.OOF
MaterialListGUI = materialListGUI.MaterialListGUI
MaterialPropGUI = materialListGUI.MaterialPropGUI

class MaterialsPage(oofGUI.MainPage):
    def __init__(self):
        oofGUI.MainPage.__init__(self, name="Materials", ordering=100,
                                 tip='Define Materials')
        pane = GtkHPaned()              # Properties on left, Materials on R.
        self.gtk.add(pane)

        self.propertypane = PropertyPane(self)
        self.materialpane = MaterialPane(self)
        pane.pack1(self.propertypane.gtk, resize=1, shrink=0)
        pane.pack2(self.materialpane.gtk, resize=1, shrink=0)
        self.sensitize()

    def current_material_name(self):
        return self.materialpane.materialName.get_value()

    # Returns a MaterialListGUI object.
    def current_material(self):
        name = self.current_material_name()
        if name:
            return self.materialpane.all_materials[name]
        return None

    # Returns a tuple, (name, property-registration-object)
    def current_property(self):
        return self.propertypane.current_property
    
    def current_property_name(self):
        prop = self.current_property()
        if prop:
            return prop[0]
        return None
    
    def sensitize(self):
        self.propertypane.sensitize()
        self.materialpane.sensitize()
        
class PropertyPane:
    def __init__(self, parent):
        debug.mainthreadTest()
        self.parent = parent
        # Property selection state lives here.  When not None,
        # current_property is a tuple, (name, propertyregistration).
        # current_property mirrors the selection state of the
        # GfxLabelTree self.propertytree.
        self.current_property = None

        self.gtk = GtkFrame('Property')
        vbox = GtkVBox(spacing=3)
        
        self.gtk.add(vbox)

        # Button box above the Property Tree
        buttonbox = GtkHButtonBox()
        buttonbox.set_layout(BUTTONBOX_SPREAD)
        buttonbox.set_spacing(0)
        buttonbox.set_child_size(0,0)
        buttonbox.set_child_ipadding(0,0)
        vbox.add(buttonbox, expand=0, fill=0)

        self.copybutton = GtkButton('Copy...')
        self.copybutton.connect('clicked', self.on_copy_property)
        guitop.top().tooltip(self.copybutton, 'Create a named copy of the currently selected property')
        buttonbox.add(self.copybutton)

        self.parambutton = GtkButton('Parametrize...')
        self.parambutton.connect('clicked', self.parametrize)
        guitop.top().tooltip(self.parambutton,
                             "Set parameters for the currently selected Property.")
        buttonbox.add(self.parambutton)
        

        self.deletebutton = GtkButton('Delete')
        self.deletebutton.connect('clicked', self.GUIdelete)
        guitop.top().tooltip(self.deletebutton, "Delete the currently selected Property.")
        buttonbox.add(self.deletebutton)



        # Scrolling window containing the Property Tree
        scroll = GtkScrolledWindow()
        vbox.add(scroll, expand=1, fill=1)
        scroll.set_policy(POLICY_ALWAYS, POLICY_ALWAYS)
        self.propertytree = gfxLabelTree.GfxLabelTree(AllProperties.data,
                                                      callback=self.proptreeCB)
        scroll.add_with_viewport(self.propertytree.gtk)


        # The Load button.
        alignbox = GtkAlignment(xa=0.5)
        labelbox = GtkHBox()
        labelbox.add(GtkLabel("Add Property to Material "), expand=0, fill=0)
        labelbox.add(GtkArrow(at=ARROW_RIGHT), expand=0, fill=0)
        self.loadbutton = GtkButton()
        alignbox.add(labelbox)
        self.loadbutton.add(alignbox)
        vbox.add(self.loadbutton, expand=0, fill=0, padding=3)
        guitop.top().tooltip(self.loadbutton,
                             "Load the currently selected property into the current material.")
        self.loadbutton.connect('clicked',self.on_prop_load)

        # Utility callbacks.
        self.propertytree.gtk.connect("button-press-event", self.on_buttonpress)

        self.propertytree.gtk.connect("destroy",self.on_destroy)

        switchboard.requestCallback("new property", self.newPropCB)

    def sensitize(self):
        debug.mainthreadTest()
        sensitivity = self.current_property is not None

        self.copybutton.set_sensitive(sensitivity)
        self.parambutton.set_sensitive(sensitivity)
        self.deletebutton.set_sensitive(sensitivity)

        matselected = self.parent.current_material_name() is not None
        self.loadbutton.set_sensitive(sensitivity and matselected)

        if sensitivity:
            mainmenu.OOF.File.Save.Property.enable()
            self.propertytree.expandToPath(self.current_property[0])
        else:
            mainmenu.OOF.File.Save.Property.disable()
        

    # Although the load button is in the Property pane, loading is
    # a material-menu operation.
    def on_prop_load(self,button):
        if self.current_property:
            propname = self.current_property[0]
            mat = self.parent.current_material()
            if mat:
                OOF.Material.Add_property(name = mat.name,
                                          property=propname)
                mat.selection_sync()
            else:
                reporter.report("No material selected for property import.")
        else:
            reporter.report("No property selected for import.")

    #######################################################################
    #
    
    # Master property selection and deselection routines, called
    # in response to both materialsPane and propertyPane selection events.

    def select_property(self, name):
        treenode = AllProperties.data[name]
        self.propertytree.selectObject(treenode.object)
        self.current_property = (name, treenode.object)
        self.sensitize()
        
    def deselect_property(self, name):
        if self.current_property: # and self.propertytree:
            if self.current_property[0]==name:
                self.propertytree.deselect()
                self.current_property = None
                self.sensitize()
            else:
                print "Inconsistent selection state."
        

    def proptreeCB(self, signal, treenode): # GfxLabelTree callback
        prop_name = treenode.path()
        if signal == "select":
            self.select_property(prop_name)
            self.parent.materialpane.select_property(prop_name)
        elif signal == "deselect":
            self.deselect_property(prop_name)
            self.parent.materialpane.deselect_property(prop_name)
        elif signal == "doubleclick":
            self.select_property(prop_name)
            self.parametrize()

    def newPropCB(self, propertyregistration):
        self.select_property(propertyregistration.name())

    ####################################################################
    #
    
    # Right-click handler for the tree -- calls the parametrize
    # button callback if it gets a right-click.
    def on_buttonpress(self, gtktree, event):  # path,object,signal):
        if (event.type == GDK.BUTTON_PRESS) and (event.button == 3):
            self.parametrize()

    # Callback for the parametrize button, brings up a dialog box to
    # fill in the parameters of the currently-selected property.
    def parametrize(self, gtkobj=None):
        if self.current_property:
            reg = self.current_property[1]
            if reg is not None:
                # Get the associated LabelTree node.
                ltn = AllProperties.data.reverse_dict[reg]
                menuitem = ltn.menus[AllProperties.parametrizekey]
                if parameterwidgets.getParameters(
                    title='Parametrize '+self.current_property[0],
                    *menuitem.params):
                    menuitem.callWithDefaults()
            else:
                reporter.report("Property is not parametrizable.")
        else:
            reporter.report("No property selected.")
            
    def on_destroy(self,gtk):
        self.gtktree = None
        
    def on_copy_property(self, button):
        menuitem = OOF.Property.Copy
        newnameparam = menuitem.get_arg('newname')
        if parameterwidgets.getParameters(
            newnameparam,
            title='Copy property ' + self.current_property[0]):
            if newnameparam.nontrivial():
                menuitem.callWithDefaults(property=self.current_property[0])

    def GUIdelete(self, gtk):
        OOF.Property.Delete(property=self.current_property[0])

########################################################################
########################################################################

class MaterialPane:
    def __init__(self, parent):
        debug.mainthreadTest()
        self.parent = parent

        # Need a list of all the material names with which to update
        # the Chooser, *and* a name-indexed dictionary of shadow
        # (MaterialListGUI) objects.
        self.all_material_names = []
        self.all_materials = {}

        # Keep track of the most recent selection, and
        # de-select it before it gets overwritten.
        self.last_selection = None
        
        # Current material name state info moved to the Chooser widget.
        # self.current_material=None

        self.gtk = GtkHBox()
      
        frame = GtkFrame('Material')
        # frame.set_usize(250,-2)
        self.gtk.add(frame, expand=1, fill=1)
        vbox = GtkVBox(spacing=3)
        frame.add(vbox)

        buttonbox = GtkHButtonBox()
        buttonbox.set_layout(BUTTONBOX_SPREAD)
        buttonbox.set_spacing(0)
        buttonbox.set_child_size(0,0)
        buttonbox.set_child_ipadding(0,0)
        vbox.add(buttonbox, padding=3, expand=0, fill=0)

        self.newmaterial = GtkButton('New...')
        buttonbox.add(self.newmaterial)
        guitop.top().tooltip(self.newmaterial, 'Create a new material.')
        
        self.copymaterial = GtkButton('Copy...') 
        buttonbox.add(self.copymaterial)
        guitop.top().tooltip(self.copymaterial,
                             'Create a copy of this material.')

        self.deletebutton = GtkButton('Delete')
        buttonbox.add(self.deletebutton)
        guitop.top().tooltip(self.deletebutton, 'Delete this material.')

        self.savebutton = GtkButton('Save...')
        buttonbox.add(self.savebutton)
        guitop.top().tooltip(self.savebutton, 'Save this material in a file.')

        self.materialName = chooser.ChooserWidget(
            self.all_materials.keys(),
            callback=self.newMatSelection,
            update_callback=self.newMatSelection)

        vbox.add(self.materialName.gtk, expand=0, fill=0)
        guitop.top().tooltip(self.materialName.gtk,
                             'Choose a Material to edit.')

        # The list of Properties belonging to a Material
        hbox = GtkHBox()
        vbox.add(hbox, padding=3, expand=1, fill=1)
        scroll = GtkScrolledWindow()
        scroll.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        hbox.add(scroll, padding=3, expand=1, fill=1)
        self.matproplist = GtkList()
        scroll.add_with_viewport(self.matproplist)

        paddingbox = GtkHBox()
        vbox.add(paddingbox, expand=0, fill=0, padding=3)

        innervbox = GtkVBox()
        paddingbox.add(innervbox, expand=1, fill=1, padding=5)
        
        self.removebutton = GtkButton('Remove Property from Material')
        innervbox.add(self.removebutton, expand=0, fill=0, padding=3)
        guitop.top().tooltip(self.removebutton,
               'Remove the currently selected property from this material.')

        # Assignment of materials to pixels and removal of materials
        # from pixels may belong in a separate GUI page.  For now,
        # it's done via the dialog boxes raised by the buttons defined
        # here.
        assignframe = GtkFrame()
        assignframe.set_shadow_type(SHADOW_IN)
        innervbox.add(assignframe, expand=0, fill=0, padding=3)
        inner2hbox = GtkHBox()
        inner2vbox = GtkVBox()
        assignframe.add(inner2hbox)
        inner2hbox.add(inner2vbox, padding=3)
        # Assign materials to pixels
        self.assignbutton = GtkButton('Assign Material to Pixels...')
        inner2vbox.add(self.assignbutton, expand=0, fill=0, padding=2)
        guitop.top().tooltip(self.assignbutton,
                             'Assign the currently selected Material to pixels in a Microstructure.')
        # Remove materials from pixels
        self.removematbutton = GtkButton('Remove Materials from Pixels...')
        inner2vbox.add(self.removematbutton, expand=0, fill=0, padding=2)
        guitop.top().tooltip(self.removematbutton,
                             'Remove all Materials from pixels in a Microstructure.')

        # GUI callbacks
        self.newmaterial.connect("clicked",self.on_newmaterial)
        self.deletebutton.connect("clicked",self.on_delete)
        self.copymaterial.connect("clicked", self.on_copy)
        self.removebutton.connect("clicked", self.on_remove)
        self.assignbutton.connect('clicked', self.on_assignment)
        self.removematbutton.connect('clicked', self.on_MS_remove_material)
        self.savebutton.connect('clicked', self.on_save)


        self.gtk.show_all()

        # Switchboard callbacks.
        switchboard.requestCallback("new_material",self.new_mat)
        switchboard.requestCallback("remove_material",self.del_mat)
        switchboard.requestCallback("prop_added_to_material",self.prop_added)
        switchboard.requestCallback("prop_removed_from_material",
                                    self.prop_removed)
        switchboard.requestCallback(('new who', 'Microstructure'),
                                    self.sensitize)


    def sensitize(self, *args):
        debug.mainthreadTest()
        mat_selected = self.materialName.get_value() is not None
        self.copymaterial.set_sensitive(mat_selected)
        self.deletebutton.set_sensitive(mat_selected)
        self.savebutton.set_sensitive(mat_selected)

        nmicros = microstructure.microStructures.nActual()
        self.assignbutton.set_sensitive(mat_selected and nmicros > 0)
        self.removematbutton.set_sensitive(nmicros > 0)

        self.removebutton.set_sensitive(
            self.current_propertyname() is not None)
            

        
    def current_propertyname(self):
        nm = self.materialName.get_value()
        if nm is not None:
            el = self.all_materials[nm].current_element
            if el is not None:
                return el.name
    
    #####################################################################
    # Material addition and deletion.
    
    def on_newmaterial(self,button):
        menuitem = OOF.Material.New
        if parameterwidgets.getParameters(title='New material',
                                          *menuitem.params):
            if menuitem.get_arg('name').nontrivial():
                menuitem.callWithDefaults()
        
    # The switchboard callback, called when a new material is made
    # by the menu.  Selects the new material.
    def new_mat(self, mat_name, mat_obj):
        self.all_material_names.append(mat_name)
        # Make the GUI shadow object for this material,
        # passing ourselves in as the parent.
        mat_shadow = MaterialListGUI(
            self, mat_name, mat_obj, self.matproplist)
        self.all_materials[mat_name]=mat_shadow

        self.materialName.update(self.all_material_names)
        # Select it.  It was appended, so it's the last item.
        self.materialName.set_state(mat_name)


    # Material deletion.
    
    # GUI callback.
    def on_delete(self,gtk):
        name = self.materialName.get_value()
        if name:
            OOF.Material.Delete(name=name)
        else:
            reporter.report("No material selected for deletion.")

    # Switchboard callback.  "material" is a soon-to-be-del'd
    # MaterialProps object.
    def del_mat(self, material):
        debug.mainthreadTest()

        mat_shadow = self.all_materials[material.name]

        self.all_material_names.remove(material.name)
        del self.all_materials[material.name]
        self.materialName.update(self.all_material_names)
    

    ########################################################################
    # Material selection and deselection.

    # Callback for the chooser.  "name" is the newly-selected name.
    def newMatSelection(self, gtkobj, name):
        # Only do the deselect if the last selection exists and has
        # a different name than the new selection.
        if self.last_selection:
            if self.last_selection.name==name:
                return
            self.last_selection.deselect()

        if name:
            self.last_selection = self.all_materials[name]
        else:
            self.last_selection = None

        # Only do the select if the new "last_selection" is nontrivial.
        if self.last_selection:
            self.last_selection.select()
            self.last_selection.selection_sync()
        self.parent.sensitize()
        
        
    #######################################################################
    # Addition and deletion of properties to/from materials.

    # Switchboard callback, called when a property has been added to
    # a material.  "material" is a MaterialProps object, "name"
    # is the fully-qualifed name from the property tree, and
    # "property" is the property registration object.
    def prop_added(self, material, name, property):
        debug.mainthreadTest()
        splitname = string.split(name,':')

        # If it's a NamedPropertyRegistration, include the class in the label.
        if hasattr(property,"parent"):
            label = splitname[-2]+':'+splitname[-1]
        else:
            label = splitname[-1]
        #
        new_prop = MaterialPropGUI(name, label=label,
                                   gui_props=self.matproplist)
        # Material in question may not be current in the GUI, so look it up.
        shadow = self.all_materials[material.name]
        shadow.add_prop(name, new_prop)
        if self.materialName.get_value() == material.name:
            self.matproplist.add(new_prop.gtk)
            new_prop.gtk.show()
            shadow.selection_sync()
        self.sensitize()

    # GUI callback, just calls the menu item.
    def on_remove(self,gtk):
        current_name = self.materialName.get_value()
        if current_name is not None:
            shadow = self.all_materials[current_name]

            if shadow.current_element:
                propname = shadow.current_element.name
                OOF.Material.Remove_property(name=current_name,
                                             property=propname)
            else:
                reporter.report("No property to remove.")
        else:
            reporter.report("No material selected.")
                
    # Switchboard callback, changes the GUI.
    # material is the MaterialProps object, name is the fully qualified
    # name from the tree, and "property" is the registration object.
    # The material may or may not be the current material.
    def prop_removed(self, material, name, property):
        gui_material = self.all_materials[material.name]
        gui_material.remove_prop(name)
        self.sensitize()




    ####################################################################
    # Selection/deselection of properties in materials.

    # These first two routines are called from the MaterialListGUI
    # object which gets the actual signal.  They relay it locally
    # and to the propertypane, so selections remain synchronized.
    def prop_select(self, propname):
        self.select_property(propname)
        self.parent.propertypane.select_property(propname)

    def prop_deselect(self, propname):
        self.deselect_property(propname)
        self.parent.propertypane.deselect_property(propname)

    # Main property selection routines, called for both local
    # selections and selections made in the property pane.
    def select_property(self, propname):
        current_name = self.materialName.get_value()
        if current_name:
            shadow = self.all_materials[current_name]
            shadow.select_prop(propname)
        self.sensitize()

    def deselect_property(self, propname):
        current_name = self.materialName.get_value()
        if current_name:
            shadow = self.all_materials[current_name]
            shadow.deselect_prop(propname)
        self.sensitize()

    #####################################################################
    # Copying.  

    def on_copy(self,button):
        current_name = self.materialName.get_value()
        if current_name:
            menuitem = OOF.Material.Copy
            newnameparam = menuitem.get_arg('newname')
            if parameterwidgets.getParameters(
                newnameparam, title="Name for the new material."):
                if newnameparam.nontrivial():
                    menuitem.callWithDefaults(name=current_name)
        else:
            reporter.report("No current material to copy.")

    #################################

    # Assignment and removal of materials from the microstructure

    def on_assignment(self, button):
        menuitem = OOF.Material.Assign
        params = filter(lambda x: x.name!='material', menuitem.params)
        materialname = self.materialName.get_value()
        if parameterwidgets.getParameters(
            title='Assign material %s to pixels' % materialname, *params):
            menuitem.callWithDefaults(material=materialname)

    def on_MS_remove_material(self, button):
        menuitem = OOF.Material.Remove
        if parameterwidgets.getParameters(
            title='Remove the assigned material from pixels',*menuitem.params):
            menuitem.callWithDefaults()

    def on_save(self, button):
        # Save a single material
        menuitem = OOF.File.Save.Materials
        params = filter(lambda x: x.name!='materials', menuitem.params)
        materialname = self.materialName.get_value()
        file = fileselector.fileSelector(
            ident='SaveMat',
            title='Save Material "%s"' % materialname,
            params=[menuitem.get_arg('format')])
        if file is not None:
            mode = fileselector.get_file_mode(file)
            if mode is not None:
                menuitem.callWithDefaults(filename=file, mode=mode,
                                          materials=[materialname])
        
# Create the page
materialspage = MaterialsPage()

#####################################

# GUI callbacks for the save-property and save-materials menu entries
# in the main FILE menu.

_filemenu = mainmenu.OOF.File
from common.IO.GUI import fileselector

# Save the current property, if selected, etc.
def _save_prop(menuitem):
    global materialspage
    propname = materialspage.current_property_name()
    if propname:
        file = fileselector.fileSelector(ident='PropMenu',
                                         title='Save Property',
                                         params=[menuitem.get_arg('format')])
        if not file is None:
            mode = fileselector.get_file_mode(file)
            if not mode is None:
                menuitem.callWithDefaults(property=propname, filename=file,
                                          mode=mode)
    else:
        reporter.report("No property selected for saving.")

mainmenu.OOF.File.Save.Property.gui_callback = _save_prop

def _save_mat(menuitem):
    global materialspage
    matname = materialspage.current_material_name()
    if matname:
        menuitem.get_arg('materials').value = [matname]
        file = fileselector.fileSelector(
            ident='MatMenu',
            title='Save Material',
            params=[menuitem.get_arg('materials'),
                    menuitem.get_arg('format')])
        if not file is None:
            mode = fileselector.get_file_mode(file)
            if not mode is None:
                menuitem.callWithDefaults(filename=file, mode=mode)
    else:
        reporter.report("No material selected for saving.")

mainmenu.OOF.File.Save.Materials.gui_callback = _save_mat
