# -*- python -*-
# $RCSfile: skeletongroupwidgets.py,v $
# $Revision: 1.35 $
# $Author: langer $
# $Date: 2004/07/28 18:32:40 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from SWIG.common import switchboard
from common import debug
from common.IO import selectionplaceholder
from common.IO.GUI import chooser
from common.IO.GUI import parameterwidgets
from common.IO.GUI import whowidget
from engine import boundarybuilder
from engine import skeletoncontext
from engine.IO import skeletongroupparams
from engine.IO import meshbdymenu
#from gtk import *
import engine.mesh

# Special widgets for the SkeletonGroup parameters.

# The "defaults" argument is used by the "Aggregate" widget, to
# provide a mechanism for picking sets of SkeletonSelectables which
# are not groups.  Currently only "<selection>" is available, but
# other examples could include "all", "none", and possibly others.
class SkeletonGroupWidget(parameterwidgets.ParameterWidget):
    def __init__(self, groups=[], defaults=[], scope=None):
        self.defaults = defaults
        self.widget = chooser.ChooserWidget(groups, self.selectCB)
        parameterwidgets.ParameterWidget.__init__(self, self.widget.gtk, scope)
        self.skelwidget = scope.findWidget(
            lambda w: isinstance(w, whowidget.WhoWidget)
            and w.whoclass is skeletoncontext.skeletonContexts)
        self.update()
        self.sbcallbacks = [
            switchboard.requestCallback(self.skelwidget, self.update),
            switchboard.requestCallback("groupset member added", self.update),
            switchboard.requestCallback("groupset member renamed", self.update),
            switchboard.requestCallback("groupset changed", self.update)
            ]
        self.setValidity(self.widget.nChoices() > 0)

    def get_value(self):
        return self.widget.get_value()

    def selectCB(self, gtkobj, result):
        pass

    def update(self, *args, **kwargs):
        try:
            skel = skeletoncontext.skeletonContexts[self.skelwidget.get_value()]
        except KeyError:
            self.redraw(None)
        else:
            self.redraw(skel)
        self.setValidity(self.widget.nChoices() > 0)

    def cleanUp(self):
        parameterwidgets.ParameterWidget.cleanUp(self)
        map(switchboard.removeCallback, self.sbcallbacks)

class SkeletonAggregateWidget(SkeletonGroupWidget):
    def get_value(self):
        rval = self.widget.get_value()
        if rval == selectionplaceholder.selectionIDstring:
            return selectionplaceholder.selectionPlaceHolder
        return rval
        

class NodeGroupWidget(SkeletonGroupWidget):
    def __init__(self, param, groups=[], scope=None):
        SkeletonGroupWidget.__init__(self, groups, scope=scope)

    def redraw(self, skeletoncontext):
        if skeletoncontext:
            self.widget.update(self.defaults +
                               skeletoncontext.nodegroups.allGroups())
        else:
            self.widget.update(self.defaults)

def _makeNodeGroupWidget(self, scope=None):   # "self" is the param instance.
    return NodeGroupWidget(self, scope=scope)

skeletongroupparams.NodeGroupParameter.makeWidget = _makeNodeGroupWidget


class SkeletonAggregateWidget(SkeletonGroupWidget):
    def get_value(self):
        rval = self.widget.get_value()
        if rval == selectionplaceholder.selectionIDstring:
            return selectionplaceholder.selectionPlaceHolder
        return rval

    
class NodeAggregateWidget(SkeletonAggregateWidget, NodeGroupWidget):
    def __init__(self, param, groups=[], scope=None):
        SkeletonGroupWidget.__init__(self, groups,
                                     defaults=[selectionplaceholder.selectionIDstring],
                                     scope=scope)
        
def _makeNodeAggregateWidget(self, scope=None):
    return NodeAggregateWidget(self, scope=scope)

skeletongroupparams.NodeAggregateParameter.makeWidget = _makeNodeAggregateWidget


####

class SegmentGroupWidget(SkeletonGroupWidget):
    def __init__(self, param, groups=[], scope=None):
        SkeletonGroupWidget.__init__(self, groups, scope=scope)

    def redraw(self, skeletoncontext):
        if skeletoncontext:
            self.widget.update(self.defaults +
                               skeletoncontext.segmentgroups.allGroups() )
        else:
            self.widget.update(self.defaults)

def _makeSegmentGroupWidget(self, scope=None):
    return SegmentGroupWidget(self, scope=scope)

skeletongroupparams.SegmentGroupParameter.makeWidget = _makeSegmentGroupWidget

class SegmentAggregateWidget(SkeletonAggregateWidget, SegmentGroupWidget):
    def __init__(self, param, groups=[], scope=None):
        SkeletonGroupWidget.__init__(self, groups,
                                     defaults=[selectionplaceholder.selectionIDstring],
                                     scope=scope)
        
def _makeSegmentAggregateWidget(self, scope=None):
    return SegmentAggregateWidget(self, scope=scope)

skeletongroupparams.SegmentAggregateParameter.makeWidget = _makeSegmentAggregateWidget

####

class ElementGroupWidget(SkeletonGroupWidget):
    def __init__(self, param, groups=[], scope=None):
        SkeletonGroupWidget.__init__(self, groups, defaults=[], scope=scope)

    def redraw(self, skeletoncontext):
        if skeletoncontext:
            self.widget.update(self.defaults +
                               skeletoncontext.elementgroups.allGroups())
        else:
            self.widget.update(self.defaults)

def _makeElementGroupWidget(self, scope=None):
    return ElementGroupWidget(self, scope=scope)

skeletongroupparams.ElementGroupParameter.makeWidget = _makeElementGroupWidget

class ElementAggregateWidget(SkeletonAggregateWidget, ElementGroupWidget):
    def __init__(self, param, groups=[], scope=None):
        SkeletonGroupWidget.__init__(self, groups,
                                     defaults=[selectionplaceholder.selectionIDstring],
                                     scope=scope)
        

def _makeElementAggregateWidget(self, scope=None):
    return ElementAggregateWidget(self, scope=scope)

skeletongroupparams.ElementAggregateParameter.makeWidget = _makeElementAggregateWidget

# ## ### #### ##### ###### ####### ######## ####### ###### ##### #### ### ## #


# Scope-aware widget for picking the boundary from the local mesh or
# skeleton context.
class ContextBoundaryWidget(parameterwidgets.ParameterWidget):
    def __init__(self, param, boundaries=[], scope=None):
        self.widget = chooser.ChooserWidget(boundaries, self.selectCB)
        parameterwidgets.ParameterWidget.__init__(self, self.widget.gtk, scope)
        self.ctxtwidget = scope.findWidget(
            lambda w: isinstance(w, whowidget.WhoWidget)
            and (w.whoclass is skeletoncontext.skeletonContexts  or
                 w.whoclass is engine.mesh.meshes))
        self.update()
        self.sbcallbacks = [
            switchboard.requestCallback(self.ctxtwidget, self.update),
            switchboard.requestCallback("new boundary configuration", self.update)
            ]
        if param.value is not None:
            self.widget.set_state(param.value)

    def get_value(self):
        return self.widget.get_value()
    def selectCB(self, gtkobj, result):
        pass

    def update(self, *args, **kwargs):
        contextval = self.ctxtwidget.get_value()
        if self.ctxtwidget.whoclass is skeletoncontext.skeletonContexts:
            try:
                context = skeletoncontext.skeletonContexts[contextval]
            except KeyError:
                context = None
        else: # meshContexts case
            try:
                context = engine.mesh.meshes[contextval]
            except KeyError:
                context = None

        self.redraw(context)
        self.setValidity(self.widget.nChoices() > 0)

    def redraw(self, context):
        if context:
            self.widget.update(context.allBoundaryNames())
        else:
            self.widget.update([])

    def cleanUp(self):
        parameterwidgets.ParameterWidget.cleanUp(self)
        map(switchboard.removeCallback, self.sbcallbacks)
        
def _makeContextBoundaryWidget(self, scope=None):
    return ContextBoundaryWidget(self, scope=scope)

skeletongroupparams.BoundaryParameter.makeWidget = _makeContextBoundaryWidget


########

# Variant, does only edge boundaries.
class ContextEdgeBoundaryWidget(ContextBoundaryWidget):
    def redraw(self, context):
        if context:
            self.widget.update(context.allEdgeBoundaryNames())
        else:
            self.widget.update([])

def _makeCEBW(self, scope=None):
    return ContextEdgeBoundaryWidget(self, scope=scope)

meshbdymenu.EdgeBoundaryParameter.makeWidget = _makeCEBW


# ## ### #### ##### ###### ####### ######## ####### ###### ##### #### ### ## #

from engine import profile

class ProfileNameWidget(parameterwidgets.ParameterWidget):
    def __init__(self, param, profiles=[], scope=None):
        self.widget = chooser.ChooserWidget(profiles, self.selectCB)
        parameterwidgets.ParameterWidget.__init__(self, self.widget.gtk, scope)
        self.update(param.value)
        # Widget needs to notice both when profiles are changed and when
        # they're renamed -- "changed" signal is for addition/removal of
        # profiles, whereas "renamed" is of course for rename operations.
        self.sbcallbacks = [
            switchboard.requestCallback("profiles changed", self.update),
            switchboard.requestCallback("profile renamed", self.update)
            ]

    def get_value(self):
        return self.widget.get_value()
    
    def selectCB(self, gtkobj, result):
        pass

    def update(self, name):
        self.widget.update(profile.AllProfiles.keys())
        if name:
            self.widget.set_state(name)
        self.setValidity(self.widget.nChoices() > 0)

    def cleanUp(self):
        parameterwidgets.ParameterWidget.cleanUp(self)
        map(switchboard.removeCallback, self.sbcallbacks)
        
def _makeProfileNameWidget(self, scope=None):
    return ProfileNameWidget(self, scope=scope)

profile.ProfileNameParameter.makeWidget = _makeProfileNameWidget
