# -*- python -*-
# $RCSfile: meshinfodisplay.py,v $
# $Revision: 1.12 $
# $Author: langer $
# $Date: 2004/09/16 19:14:18 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov.

from common import color
from common import primitives
from common import registeredclass
from common.IO import display
from common.IO import parameter

class MeshInfoDisplay(display.DisplayMethod):
    def __init__(self, querycolor, peekcolor, node_size,
                 element_width):
        self.querycolor = querycolor
        self.peekcolor = peekcolor
        self.colors = {"query": self.querycolor, "peek": self.peekcolor}
        self.node_size = node_size
        self.element_width = element_width
        display.DisplayMethod.__init__(self)
        self.drawFuncs = {"Element": self.drawElement,
                          "Node": self.drawNode}

    def draw(self, gfxwindow, device):
        toolbox = gfxwindow.getToolboxByName("Mesh_Info")
        # Drawing "queried" item.
        if toolbox.querier and toolbox.querier.object:
            self.drawFuncs[toolbox.querier.targetname](device,
                                                       toolbox.querier.object,
                                                       which="query")
        # Drawing "peeked" item.
        if toolbox.peeker and toolbox.peeker.objects.values():
            for objtype in toolbox.peeker.objects.keys():
                if toolbox.peeker.objects[objtype]:
                    self.drawFuncs[objtype](device,
                                            toolbox.peeker.objects[objtype],
                                            which="peek")

    def drawElement(self, device, element, which="query"):
        device.set_lineColor(self.colors[which])
        device.set_lineWidth(self.element_width)
        nlist = [node for node in element.node_iterator()]
        for i in range(len(nlist)):
            n0 = nlist[i]
            n1 = nlist[(i+1)%len(nlist)]
            device.draw_segment(primitives.Segment(n0.position(),
                                                   n1.position()))

    def drawNode(self, device, node, which="query"):
        device.set_lineColor(self.colors[which])
        device.set_lineWidth(self.node_size)
        device.draw_dot(node.position())        

    def getTimeStamp(self, gfxwindow):
        toolbox = gfxwindow.getToolboxByName("Mesh_Info")
        if toolbox.querier and toolbox.peeker:
            return max(self.timestamp,
                       toolbox.querier.getTimeStamp(),
                       toolbox.peeker.getTimeStamp())
        elif toolbox.querier and not toolbox.peeker:
            return max(self.timestamp,
                       toolbox.querier.getTimeStamp())
        else:
            return self.timestamp
                
# This object should be created via the registration, and not
# directly via the initializer, because the registration creation
# method gives it a timestamp.

meshInfoDisplay = registeredclass.Registration(
    'Info',
    display.DisplayMethod,
    MeshInfoDisplay,
    params=[color.ColorParameter('querycolor', color.RGBColor(0.0, 0.5, 1.0)),
            color.ColorParameter('peekcolor', color.RGBColor(1.0, 0.5, 0.5)),
            parameter.IntRangeParameter('node_size', (0,10), 3),
            parameter.IntRangeParameter('element_width', (0,10), 2)],
    ordering=4.0,
    layerordering=display.PointLike(100),
    whoclasses=('Mesh',),
    tip="Set display parameters for the decorations used by the Mesh Info toolbox.")

display.predefineLayer('Mesh', '<topmost>', meshInfoDisplay())
