# -*- python -*-
# $RCSfile: pypropertywrapper_.py,v $
# $Revision: 1.8 $
# $Author: reida $
# $Date: 2003/04/02 14:52:31 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

# These are the Pythonized wrappers for the virtual functions
# of the Property class, as seen from Python.  They are called
# back from the C++ PyPropertyWrapper class, and in turn call
# the corresponding methods of the derived pure-Python
# property.  Their primary function is to wrap their arguments
# in <whatever>Ptr constructors, so all the SWIG'd methods
# are available in the derived-class routine.

from SWIG.engine import elements

# These defs could be done by pragma's in the SWIG file, but
# having all the names be local is good.
from SWIG.engine import femesh
FEMeshPtr = femesh.FEMeshPtr
from SWIG.engine import elementnodeiterator
ElementFuncNodeIteratorPtr = elementnodeiterator.ElementFuncNodeIteratorPtr
from SWIG.engine import mastercoord
MasterPositionPtr = mastercoord.MasterPositionPtr
from SWIG.engine import flux
FluxPtr = flux.FluxPtr
from SWIG.engine import material
MaterialPtr = material.MaterialPtr

def _PyPropertyWrapper_precomputewrap(self):
    if hasattr(self,'precompute'):
        self.precompute()

PyPropertyWrapperPtr.precomputewrap = _PyPropertyWrapper_precomputewrap

def _PyPropertyWrapper_cross_referencewrap(self, material_p):
    if hasattr(self,'cross_reference'):
        material = MaterialPtr(material_p)
        self.cross_reference(material)

PyPropertyWrapperPtr.cross_referencewrap = _PyPropertyWrapper_cross_referencewrap


def _PyPropertyWrapper_begin_elementwrap(self, element_p):
    if hasattr(self,'begin_element'):
        elmnt = elements.ElementPtr(element_p)
        self.begin_element(elmnt)

PyPropertyWrapperPtr.begin_elementwrap = _PyPropertyWrapper_begin_elementwrap


def _PyPropertyWrapper_end_elementwrap(self, element_p):
    if hasattr(self,'end_element'):
        elmnt = elements.ElementPtr(element_p)
        self.end_element(elmnt)

PyPropertyWrapperPtr.end_elementwrap = _PyPropertyWrapper_end_elementwrap


def _PyPropertyWrapper_post_processwrap(self, mesh_p, element_p):
    if hasattr(self,'post_process'):
        mesh = FEMeshPtr(mesh_p)
        elmnt = elements.ElementPtr(element_p)
        self.post_process(mesh, elmnt)

PyPropertyWrapperPtr.post_processwrap = _PyPropertyWrapper_post_processwrap


def _PyPropertyWrapper_fluxmatrixwrap(self, mesh_p, element_p,
                                      efni_p, flux_p, masterpos_p):
    if hasattr(self,'fluxmatrix'):
        mesh = FEMeshPtr(mesh_p)
        elmnt = elements.ElementPtr(element_p)
        efni = ElementFuncNodeIteratorPtr(efni_p)
        flux = FluxPtr(flux_p)
        masterpos = MasterPositionPtr(masterpos_p)
        self.fluxmatrix(mesh, elmnt, efni, flux, masterpos)
        
PyPropertyWrapperPtr.fluxmatrixwrap = _PyPropertyWrapper_fluxmatrixwrap

    
def _PyPropertyWrapper_fluxrhswrap(self, mesh_p, element_p,
                                  flux_p, masterpos_p):
    if hasattr(self,'fluxrhs'):
        mesh = FEMeshPtr(mesh_p)
        elmnt = elements.ElementPtr(element_p)
        flux = FluxPtr(flux_p)
        masterpos = MasterPositionPtr(masterpos_p)
        self.fluxrhs(mesh, elmnt, flux, masterpos)

PyPropertyWrapperPtr.fluxrhswrap = _PyPropertyWrapper_fluxrhswrap

# Utility functions to support storing data in an element.
def _PyPropertyWrapper_data_to_element(self, element, name, data):
    pass

# Sensible display string for element data.
def _PyPropertyElementData___repr__(self):
    return "PyPropertyElementData(%s,%s)" % (self.name(), `self.data()`)

PyPropertyElementDataPtr.__repr__ = _PyPropertyElementData___repr__
