// -*- C++ -*-
// $RCSfile: solverstepper.h,v $
// $Revision: 1.6 $
// $Author: redwing $
// $Date: 2004/09/16 15:22:18 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>
#ifndef SOLVERSTEPPER_H
#define SOLVERSTEPPER_H
class CSRmatrix;
class FEMesh;
class Solver;

double dot_product (VECTOR_D &left, VECTOR_D &right);

class SolverStepperCore {
  /*
    The base SolverStepperCore class applies a solver 
    to an femesh.
    precompute() creates the different bits, using the FEMesh.
    solve() handles the solver.
    postcompute() places the final solution in the femesh and 
    cleans whatever mess was left behind.
  */

protected:
  Solver * solver;
  FEMesh * femesh;
public:
  void clear_memory(); 
  CSRmatrix * A;
  CSRmatrix * C;
  VECTOR_D * x;
  VECTOR_D * rhs;
  VECTOR_D * xp;
  
  SolverStepperCore(Solver *, FEMesh *);
  virtual ~SolverStepperCore();
  virtual double residual();
  virtual void precompute();//create C-objects that will only persist during itertative process
  virtual bool solve();//do real work
  virtual void postcompute();//clean-up the mess


};

class NewtonSolverCore : protected SolverStepperCore {
protected:
  VECTOR_D * psi;
  VECTOR_D * dx;
public:
  virtual double residual();
  NewtonSolverCore(Solver *, FEMesh *);
  virtual ~NewtonSolverCore();
  virtual void precompute();//create C-objects that will persist during iterative process
  virtual bool solve();//do real work
  virtual void postcompute();//clean-up the mess


};
#endif // SOLVER_H
